import 'dart:async';

import 'package:flutter/foundation.dart';

import '../app.dart';
import 'adapters/http_adapter.dart';
import 'api_service.dart';
import 'interceptors.dart';
import 'utils/dio_helper.dart';
import 'utils/nullable_string_helper.dart';
import 'utils/words.dart';

export 'package:cross_file/cross_file.dart';
export 'package:dio/dio.dart';

export 'utils/dio_exception_helper.dart';

part 'api_response.dart';
part 'optional.dart';
part 'query_gql.dart';
part 'query_rest.dart';
part 'quick_api.dart';

//const userAgentHeader = "user-agent";

class API extends ApiService {
  API(
      {this.apiKey,
      String baseUrl = '',
      Duration timeout = const Duration(seconds: 30)})
      : super(
            dio: Dio(BaseOptions(
          baseUrl: baseUrl,
          connectTimeout: timeout,
          sendTimeout: timeout,
          contentType: Headers.jsonContentType,
          // headers: {userAgentHeader: 'VIQCoreFlutter'},
        ))) {
    dio
      ..allowSelfSignedCert()
      ..allowWithCredential()
      ..enableCookieManager()
      ..interceptors.add(ApiInterceptor());

    if (kDebugMode) {
      dio.interceptors.add(LogInterceptor(
          requestHeader: true,
          responseHeader: true,
          requestBody: true,
          responseBody: true));
    }
  }

  final String? apiKey;

  // static const String refreshTokenKey = 'x-refresh-token';
  // String? get refreshToken => App.box.get('api/$refreshTokenKey');
  // set refreshToken(String? token) {
  //   if (token == null) {
  //     App.box.delete('api/$refreshTokenKey');
  //   } else {
  //     App.box.put('api/$refreshTokenKey', token);
  //   }
  // }

  // void clearAuthCache() {
  //   //writeRefreshToken(null);
  //   refreshToken = null;
  //   dio.clearCookies();
  // }

  Interceptors get interceptors => dio.interceptors;

  // ignore_for_file: always_put_control_body_on_new_line

  /// Join parts of a url path into a url
  ///
  /// Typically part1 will be a http://xxx type prefix
  /// and part2-part(n) are paths to be appended to the part1
  /// When concatenating parts the '/' separator will be stripped or
  /// added as necessary to unsure there is just one '/' between parts.
  String urlJoin(String part1, String part2,
      [String? part3,
      String? part4,
      String? part5,
      String? part6,
      String? part7,
      String? part8,
      String? part9]) {
    var endpoint = _urlStrip(part1);

    endpoint = _urlJoin2(endpoint, part2);
    if (part3 != null) endpoint = _urlJoin2(endpoint, part3);
    if (part4 != null) endpoint = _urlJoin2(endpoint, part4);
    if (part5 != null) endpoint = _urlJoin2(endpoint, part5);
    if (part6 != null) endpoint = _urlJoin2(endpoint, part6);
    if (part7 != null) endpoint = _urlJoin2(endpoint, part7);
    if (part8 != null) endpoint = _urlJoin2(endpoint, part8);
    if (part9 != null) endpoint = _urlJoin2(endpoint, part9);
    return endpoint;
  }

  String _urlJoin2(String part1, String part2) =>
      '${_urlStrip(part1)}/${_urlStrip(part2)}';

  String _urlStrip(String part1) {
    String endpoint;
    if (part1.startsWith('/')) {
      endpoint = part1.substring(1);
    } else {
      endpoint = part1;
    }
    if (endpoint.endsWith('/')) {
      endpoint = endpoint.substring(0, endpoint.length - 1);
    }
    return endpoint;
  }
}
